-- An implimentation of the Minimal Deceptive Problem
--
-- Author	Matthew Caryl
-- Created	12.11.96

with Ada.Text_IO, Ada.Command_Line, Ada.Text_IO.Integer_IO, Ada.Strings;

procedure Blues is
	type Long is new Long_Integer;
	type Percentage is range 0..100;
	type Evaluation is range 1..100;
	type Chromosome is array (0..8) of Boolean;
	type Population is array (Integer range <>) of Chromosome;
	type Evaluations is array (Integer range <>) of Evaluation;

	mutation : Percentage;
	crossover : Percentage;
	red : Evaluation;
	blue : Evaluation;
	purple : Evaluation;
	black : Evaluation;
	seed : Long;

	function Random(n : in Integer) return Integer is
	begin
		seed := seed * 1103515245 + 12345;
		return (Integer(seed / 65535) mod n);
	end Random;

	function Chromo return Chromosome is
		c : Chromosome;
	begin
		for i in 0..8 loop
			c(i) := (Random(2) = 0);
		end loop;
		return c;
	end Chromo;

	function Chromo(p : Chromosome) return Chromosome is
		c : Chromosome;
		m : Integer;
	begin
		m := Random(8);
		for i in 0..8 loop
			c(i) := p(i);
		end loop;
		c(m) := not c(m);
		return c;
	end Chromo;

	function Chromo(p1, p2 : Chromosome) return Chromosome is
		c : Chromosome;
		k : Integer;
	begin
		k := Random(8);
		for i in 0..(k - 1) loop
			c(i) := p1(i);
		end loop;
		for i in k..8 loop
			c(i) := p2(i);
		end loop;
		return c;
	end Chromo; 

	function Evaluate(c : Chromosome) return Evaluation is
	begin
		if c(0) then
			if c(1) then
				return blue;
			else
				return black;
			end if;
		else
			if c(1) then
				return purple;
			else
				return red;
			end if;
		end if;
	end Evaluate;

	function Roulette(evals : Evaluations; total : Integer) return Integer is
		n : Integer;
		sum : Integer;
		i : Integer;
	begin
		n := Random(total);
		sum := 0;
		i := 0;
		while n >= sum + Integer(evals(i)) loop
			sum := sum + Integer(evals(i));
			i := i + 1;
		end loop;
		return i;
	end Roulette;

	package Long_IO is new Ada.Text_IO.Integer_IO(Long);
	package Percentage_IO is new Ada.Text_IO.Integer_IO(Percentage);
	package Evaluation_IO is new Ada.Text_IO.Integer_IO(Evaluation);

	use Ada.Text_IO, Ada.Command_Line, Percentage_IO, Long_IO, Evaluation_IO;
	procedure Input(mutation, crossover : out Percentage; red, blue, purple, black : out Evaluation; seed : out Long) is
		Input_Error : exception;
		last : Positive;
	begin
		if Argument_Count /= 7 then
			raise Input_Error;
		end if;
		Get(Argument(1), mutation, last);
		Get(Argument(2), crossover, last);
		Get(Argument(3), red, last);
		Get(Argument(4), blue, last);
		Get(Argument(5), purple, last);

		Get(Argument(6), black, last);
		Get(Argument(7), seed, last);
	exception
		when Constraint_Error | Input_Error =>
			Put_Line("Error: incorrect parameters");
			Put_Line("Correct format: " & Command_Name & " (mutation) (crossover) (red) (blue) (purple) (black) (seed)");
			Put_Line("where");
			Put_Line("mutation, crossover are an integer percentage between 0 and 100");
			Put_Line("red, blue, purple, black are positive integers");
			Put_Line("seed is a positive integer");
	end Input;
	
	use Ada.Text_IO, Evaluation_IO;
	procedure Output(pop : in Population; name : String) is
		file : File_Type;
	begin
		Create(file, Out_File, name);
		for i in pop'Range loop
			Put(file, Evaluate(pop(i)));
			New_Line(file);
		end loop;
		Close(file);
	end Output;
	
	width : constant Integer := 64;
	height : constant Integer := 8;
	pop : Population(0..height);
	offspring : Population(0..height);
	eval : Evaluations(0..height);
	total : Integer;
	r : Percentage;
	file : File_Type;
	
begin
	Input(mutation, crossover, red, blue, purple, black, seed);

	Create(file, Out_File, "chromosome.txt");
	
	-- initialise
	for i in pop'Range loop
		pop(i) := Chromo;
	end loop;

	-- generation loop
	for generation in 0..width loop
		-- evaluate
		total := 0;
		for i in 0..height loop
			eval(i) := Evaluate(pop(i));
			total := total + Integer(eval(i));
		end loop;

		-- breed
		for i in 0..height loop
			r := Percentage(Random(100));

			if (r < mutation) then
				offspring(i) := Chromo(pop(i));
			elsif (r < mutation + crossover) then
				offspring(i) := Chromo(pop(i), pop(Roulette(eval, total)));
			else
				offspring(i) := pop(i);
			end if;
		end loop;

		-- replace
		pop := offspring;
		
		-- save
		for i in pop'Range loop
			Put(file, Evaluate(pop(i)));
			New_Line(file);
		end loop;
	end loop;

	Close(file);
end Blues;
