--
-- MUTANTS : a generic genetic algorithm toolkit for Ada 95
--
-- Music markup for stringed instruments
--
-- Author  Matthew Caryl
-- Created 21.2.96
--

with Mutants.Core.Random;
with Mutants.Core.Ending.Count;
with Mutants.Core.Representation;
with Mutants.Core.Representation.Bit_Vector;
with Mutants.Core.Evaluation;
with Music_Evaluation;
with Mutants.Core.Individual;
with Mutants.Core.Population;
with Mutants.Core.Fitness.Unity;
with Mutants.Core.Selector.Roulette;
with Mutants.Core.Selector.High;
with Mutants.Core.Creator;
with Mutants.Core.Creator.Bit_Random_Initialise;
with Mutants.Core.Creator.Bit_Mutation_Reproduction;
with Mutants.Core.Creator.Vector_Bit_Random_Initialise;
with Mutants.Core.Creator.Single_Bit_Vector_Reproduction;
with Mutants.Core.Creator.Bxor;
with Mutants.Core.Creator.Bit_Onepoint_Vector_Crossover_Reproduction;
with Mutants.Core.Model.Generation;
use Mutants.Core.Random;
use Mutants.Core.Ending.Count;
use Mutants.Core.Representation;
use Mutants.Core.Representation.Bit_Vector;
use Mutants.Core.Evaluation;
use Music_Evaluation;
use Mutants.Core.Individual;
use Mutants.Core.Population;
use Mutants.Core.Fitness.Unity;
use Mutants.Core.Selector.Roulette;
use Mutants.Core.Selector.High;
use Mutants.Core.Creator;
use Mutants.Core.Creator.Bit_Random_Initialise;
use Mutants.Core.Creator.Bit_Mutation_Reproduction;
use Mutants.Core.Creator.Vector_Bit_Random_Initialise;
use Mutants.Core.Creator.Single_Bit_Vector_Reproduction;
use Mutants.Core.Creator.Bit_Onepoint_Vector_Crossover_Reproduction;
use Mutants.Core.Creator.Bxor;
use Mutants.Core.Model.Generation;

with Ada.Text_IO.Integer_IO;
with Ada.Text_IO.Float_IO;

procedure Music is

	package Integer_IO is new Ada.Text_IO.Integer_IO(Integer);
	package Float_IO is new Ada.Text_IO.Float_IO(Float);
	
	use Integer_IO;
	use Ada.Text_IO;
	procedure PrintBest(model : Model_Class; e : Music_Evaluation.Music_Evaluation) is
		p : Population_Access;
		f : Unity_Fitness;
		s : High_Selector;
		i : Individual_Access;
		r : Representation_Access;
		l : Natural;
	begin
		Get_Population(model, p);
		EvaluatePopulation(p, e);
		Attach(f, p);
		Attach(s, f);
		Get(s, i);
		r := Mutants.Core.Individual.Representation(i);
		PlayMusic(e, Vector_Representation_Access(r).all, l);
		Put(l);
		New_Line;
		PrintMusic(e, Vector_Representation_Access(r).all);
		New_Line;
	end PrintBest;

	use Integer_IO;
	use Ada.Text_IO;
	procedure ReadMusic(name : String; evaluator : out Music_Evaluation_Access; size : out Natural) is
		file : File_Type;
		number_notes : Natural;
		evaluation : Music_Evaluation_Access;
		bow_length : Natural;
		note_length : Natural;
		rule : Character;
	begin
		Open(file, In_File, name);
		Get(file, bow_length);
		Skip_Line(file);
		Get(file, number_notes);
		Skip_Line(file);
		evaluation := new Music_Evaluation.Music_Evaluation(number_notes);
		SetBowLength(evaluation.all, bow_length);
		for i in 1..number_notes loop
			Get(file, note_length);
			Get(file, rule);
			while rule = ' ' loop
				Get(file, rule);
			end loop;
			SetNote(evaluation.all, i, note_length, rule);
			Skip_Line(file);
		end loop;
		Close(file);
		evaluator := evaluation;
		size := number_notes;
	end ReadMusic;

	size : Natural := 10;
	rand : aliased Random;
	bit_initialiser : aliased Mutants.Core.Creator.Bit_Random_Initialise.Bit_Random_Initialise(rand'Unchecked_Access);
	fitness : Unity_Fitness;
	selector : Roulette_Selector(rand'Unchecked_Access);
	bit_mutation : aliased Mutants.Core.Creator.Bit_Mutation_Reproduction.Bit_Mutation_Reproduction;
	vector_mutation : aliased Mutants.Core.Creator.Single_Bit_Vector_Reproduction.Single_Vector_Reproduction(bit_mutation'Unchecked_Access, rand'Unchecked_Access);
	vector_crossover : aliased Mutants.Core.Creator.Bit_Onepoint_Vector_Crossover_Reproduction.Onepoint_Vector_Crossover_Reproduction(rand'Unchecked_Access);
	reproduction : Xor_Creator(vector_mutation'Unchecked_Access, 1, vector_crossover'Unchecked_Access, 3, rand'Unchecked_Access);
	ending : Count_Ending;
	evaluator : Music_Evaluation_Access;
	music_size : Natural;
	initialiser : Creator_Access;
	model : Model_Access;
begin
	-- initialise
	ReadMusic("music.txt", evaluator, music_size);
	initialiser := new Vector_Bit_Random_Initialise.Vector_Initialise(bit_initialiser'Unchecked_Access, music_size);
	model := new Generation_Model(initialiser, size);

	-- best
	PrintBest(model.all, evaluator.all);

	-- evolve
	for i in 1..100 loop
		Reset(ending, 1);
		Run(model.all, evaluator.all, fitness, selector, reproduction, ending);
		PrintBest(model.all, evaluator.all);
	end loop;
end Music;
