--
-- MUTANTS : A generic genetic algorithm toolkit for Ada 95
--
-- Music Evaluation component
--
-- Author  Matthew Caryl
-- Created 21.2.97
--

with Mutants.Core.Evaluation;
with Mutants.Core.Representation;
with Mutants.Core.Representation.Bit_Vector;

use Mutants.Core.Evaluation;
use Mutants.Core.Representation;
use Mutants.Core.Representation.Bit_Vector;

package Music_Evaluation is

	type Music_Evaluation(size : Integer) is new Mutants.Core.Evaluation.Evaluation with private;
	type Music_Evaluation_Access is access all Music_Evaluation;

	function Evaluate(Evaluation : in Music_Evaluation; Representation : in Representation_Access) return Float;
	procedure PlayMusic(Eval : in Music_Evaluation; Rep : in out Vector_Representation; Length : out Natural);
	procedure PrintMusic(Eval : in Music_Evaluation; Rep : in out Vector_Representation);
	
	procedure SetBowLength(Evaluation : in out Music_Evaluation; Length : in Natural);
	procedure SetNote(Evaluation : in out Music_Evaluation; Which : Natural; Length : in Natural; Rule : Character);

private
	
	type Note is
		record
			Length : Natural := 0;
			Rule : Character := 'N';
		end record;

	type Note_Array is array (Natural range <>) of Note;

	type Music_Evaluation(size : Integer) is new Mutants.Core.Evaluation.Evaluation with 
		record
			Notes : Note_Array(1..size);
			BowLength : Natural := 100;
		end record;

end Music_Evaluation;
