--
-- MUTANTS : a generic genetic algorithm toolkit for Ada 95
--
-- Core Toolkit - Xor creator component
--
-- Author  Matthew Caryl
-- Created 15.1.97
--

with Mutants.Core.Selector;
with Mutants.Core.Population;
with Mutants.Core.Random;
use Mutants.Core.Random;

package body Mutants.Core.Creator.Bxor is

	procedure Create(Creator : in out Xor_Creator; Pop : in out Mutants.Core.Population.Population; Size : in Natural) is
		t : Natural := Natural(Size / (Creator.LeftWeight + Creator.RightWeight));
		l : Natural := Creator.LeftWeight * t;
		r : Natural := Creator.RightWeight * t;
	begin
		Create(Creator.Left.all, Pop, l);
		Create(Creator.Right.all, Pop, r);
		for i in 1..(Size - l - r) loop
			if Value(Creator.Chance, Creator.LeftWeight + Creator.RightWeight) < Creator.LeftWeight then
				Create(Creator.Left.all, Pop, 1);
			else
				Create(Creator.Right.all, Pop, 1);
			end if;
		end loop;
	end Create;

	procedure Selector(Creator : in out Xor_Creator; Sel : Selector_Access; Required : out Boolean) is
		r1, r2 : Boolean;
	begin
		Selector(Creator.Left.all, Sel, r1);
		Selector(Creator.Right.all, Sel, r2);
		Required := r1 or r2;
	end Selector;
	
	function Parents(Creator : in Xor_Creator) return Natural is
		l : Integer := Parents(Creator.Left.all);
		r : Integer := Parents(Creator.Right.all);
	begin
		if l > r then
			return l;
		else
			return r;
		end if;
	end Parents;
	
	procedure Breed(Creator : in out Xor_Creator; Reps_In : in Representation_Access_Array; Rep_Out : out Representation_Access; Success : out Boolean) is
	begin
		if Value(Creator.Chance, Creator.LeftWeight + Creator.RightWeight) < Creator.LeftWeight then
			Breed(Creator.Left.all, Reps_In, Rep_Out, Success);
		else
			Breed(Creator.Right.all, Reps_In, Rep_Out, Success);
		end if;
	end Breed;	
	
end Mutants.Core.Creator.Bxor;
