--
-- MUTANTS : a generic genetic algorithm toolkit for Ada 95
--
-- Core Toolkit - Population component
--
-- Author  Matthew Caryl
-- Created 14.12.96
--

with Mutants.Core.Individual;
with Ada.Finalization;
with Ada.Unchecked_Deallocation;

use Mutants.Core.Individual;

package body Mutants.Core.Population is

-- WATCH OUT FOR ARRAY SIZES OTHER THAN 1..X

	procedure Free is new Ada.Unchecked_Deallocation(Individual_Access_Array, Individual_Access_Array_Access);
	
	procedure Add(Pop : in out Population; Individual : in Individual_Access) is
		new_individuals : Individual_Access_Array_Access;
	begin
		if Pop.Size >= Pop.Individuals'Last then
			new_individuals := new Individual_Access_Array(1..Integer(Pop.Size * 1.5));
			new_individuals(1..Pop.Size) := Pop.Individuals(1..Pop.Size);
			Free(Pop.Individuals);
			Pop.Individuals := new_individuals;
		end if;
		Pop.Size := Pop.Size + 1;
		Pop.Individuals(Pop.Size) := Individual;
	end Add;
	
	procedure Add(Pop : in out Population; Individuals : in Individual_Access_Array) is
	begin
		-- INEFFICIENT IMPLEMENTATION
		for i in 1..Individuals'Last loop
			Add(Pop, Individuals(i));
		end loop;
	end Add;
	
	procedure Add(Pop : in out Population; New_Population : Population) is
	begin
		-- INEFFICIENT IMPLEMENTATION
		for i in 1..New_Population.Size loop
			Add(Pop, New_Population.Individuals(i));
		end loop;
	end Add;
	
	procedure Sub(Pop : in out Population; Individual : in Individual_Access) is
		p, q : Integer;
	begin
		p := 1;
		q := Pop.Size + 1;
		while (p /= q) loop
			if Pop.Individuals(p) = Individual then
				q := p;
			else
				p := p + 1;
			end if;
		end loop;
		if p = Pop.Size + 1 then
			return;
		end if;
		Pop.Individuals(p..Pop.Size - 1) := Pop.Individuals(p + 1..Pop.Size);
		Pop.Size := Pop.Size - 1;
	end Sub;
	
	function Size(Pop : in Population) return Natural is
	begin
		return Pop.Size;
	end Size;
	
	procedure Copy(First, Second : in out Population) is
	begin
		Clear(First);
		Add(First, Second);
	end Copy;

	procedure Clear(Pop : in out Population) is
	begin
		Pop.Size := 0;
	end Clear;

	procedure Adjust(Pop : in out Population) is
		old_individuals : Individual_Access_Array_Access;
	begin
		old_individuals := Pop.Individuals;
		Pop.Individuals := new Individual_Access_Array(1..Pop.Size);
		Pop.Individuals(1..Pop.Size) := old_individuals(1..Pop.Size);
	end Adjust;

	procedure Finalize(Pop : in out Population) is
	begin
		Free(Pop.Individuals);
	end Finalize;

	function Iterator(Pop : in Population_Access) return PopulationIterator is
		it : PopulationIterator;
	begin
		it.Source := Pop;
		it.Place := 0;
		return it;
	end Iterator;
	
	procedure Next(Iterator : in out PopulationIterator; More : out Boolean; Individual : out Individual_Access) is
	begin
		Iterator.Place := Iterator.Place + 1;
		if Iterator.Place > Iterator.Source.Size then
			More := false;
			return;
		else
			More := true;
			Individual := Iterator.Source.Individuals(Iterator.Place);
			return;
		end if;
	end Next;

end Mutants.Core.Population;
