--
-- MUTANTS : A generic genetic algorithm toolkit for Ada 95
--
-- Core toolkit - Vector representation component
--
-- Author  Matthew Caryl
-- Created 11.12.96
--

generic
	type Element_Type is new Representation with private;
	type Element_Type_Access is access all Element_Type;

package Mutants.Core.Representation.Vector is

	type Vector_Representation(Size : Positive) is new Representation with private;
	
	function Equal(one, two : Vector_Representation) return Boolean;
	
	function Get(Representation : in Vector_Representation; Index : Positive) return Element_Type;
	procedure Set(Representation : in out Vector_Representation; Index : Positive; Element : in Element_Type);
	function Size(Representation : in Vector_Representation) return Positive;

	procedure Get(Representation : in out Vector_Representation; Index : Positive; Element : out Element_Type_Access);

	type Vector_Representation_Access is access all Vector_Representation;

	function Get(Representation : in Vector_Representation_Access; Index : Positive) return Element_Type;
	procedure Set(Representation : in Vector_Representation_Access; Index : Positive; Element : in Element_Type);
	function Size(Representation : in Vector_Representation_Access) return Positive;
	procedure Get(Representation : in Vector_Representation_Access; Index : Positive; Element : out Element_Type_Access);

private

	type Element_Vector_Array is array (Integer range <>) of aliased Element_Type; 

	type Vector_Representation(Size : Positive) is new Representation with
		record
			Element_Vector : Element_Vector_Array(1..Size);
		end record;

end Mutants.Core.Representation.Vector;
