--
-- MUTANTS : A generic genetic algorithm toolkit for Ada 95
--
-- TSP Evaluation component
--
-- Author  Matthew Caryl
-- Created 4.2.97
--

with Mutants.Core.Representation.Integer;
with Mutants.Extended.Representation.Integer_Order;
use Mutants.Core.Representation.Integer;
use Mutants.Extended.Representation.Integer_Order;

package body TSP_Evaluation is

	function Evaluate(Evaluation : in TSP_Evaluation; Representation : in Representation_Access) return Float is
		total : Float := 0.0;
		first, second : Integer_Representation;
		v : Order_Representation_Access := Order_Representation_Access(Representation);
	begin
		for i in 1..(Size(v) - 1) loop
			first := Get(v, i);
			second := Get(v, i + 1);
			total := total + Get(Evaluation, Get(first), Get(second));
		end loop;
		first := Get(v, 1);
		second := Get(v, Size(v));
		total := total + Get(Evaluation, Get(first), Get(second));
		return total;
	end Evaluate;

	procedure Put(Evaluation : in out TSP_Evaluation; Value : Float; X, Y : Integer) is
	begin
		Evaluation.matrix(x, y) := value;
	end Put;

        function Get(Evaluation : in TSP_Evaluation; X, Y : Integer) return Float is
	begin
		return Evaluation.matrix(x, y);
	end Get;

        function Size(Evaluation : in TSP_Evaluation) return Integer is
	begin
		return Evaluation.matrix'Last(1) - Evaluation.matrix'First(1) + 1;
	end Size;

end TSP_Evaluation;
